import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * Objet utilis pour la dterminisation
 * @author bfonta35
 */
public class Tables {
	/**
	 * Table associant un {@link Set ensemble} d'{@link EtatND} et
	 * un {@link EtatD}
	 */
	Map<Set<EtatND>, EtatD> table;
	
	/**
	 * Table associant un {@link EtatD} et
	 * un {@link Set ensemble} d'{@link EtatND}
	 */
	Map<EtatD, Set<EtatND>> inverse;
	
	/**
	 * Id du prochain tat ajout
	 */
	private int nextId;
	
	/**
	 * Cre des tables vierges
	 */
	Tables() {
		table = new HashMap<Set<EtatND>, EtatD>();
		inverse = new HashMap<EtatD, Set<EtatND>>();
		nextId = 0;
	}
	
	
	/**
	 * Si l'ensemble <code>s</code> n'est pas encore prsent,
	 * l'associe  un nouvel tat portant <code>{@link Tables#nextId nextId}</code>
	 * comme identifiant, ensuite incrmente <code>{@link Tables#nextId nextId}</code>
	 * et renvoit <code>true</code>. Sinon, il ne fait rien et renvoit <code>false</code>.
	 * @return <code>true</code> si l'ensemble n'est pas encore prsent,<br>
	 * <code>false</code> sinon
	 */
	boolean add(Set<EtatND> s) {
		if (table.containsValue(s)) {
			//on a deja compte cet ensemble
			return false;
		}
		//ajout
		EtatD e = new EtatD(nextId);
		if (nextId == 0) {
			e.estInit = true;
		}
		
		//on parcours les etats
		Iterator<EtatND> parcours = s.iterator();
		EtatND courant;
		
		while(parcours.hasNext()) {
			courant = parcours.next();
			if (courant.estTerm) {
				//si l'un d'eux est terminal, l'etat "determinis" l'est
				e.estTerm = true;
			}
		}
		
		table.put(s, e);
		inverse.put(e, s);
		
		this.nextId++;
		return true;
	}
	
	/**
	 * Renvoit l'tat correspondant  <code>s</code>.
	 * @param s (<code>Set&lt;{@link EtatD}&gt;)
	 * @return <code>{@link EtatD}<br>
	 * <code>null</code> si aucun tat ne correspond
	 */
	EtatD getD(Set<EtatND> s) {
		if (table.containsKey(s)) {
			return table.get(s);
		}
		return null;
	}
	
	/**
	 * Renvoit l'ensemble correspondant  <code>e</code>.
	 * @param e ({@link EtatD})
	 * @return <code>Set&lt;{@link EtatND}&gt;<br>
	 * <code>null</code> si aucun ensemble ne correspond
	 */
	Set<EtatND> getND(EtatD e) {
		if (inverse.containsKey(e)) {
			return inverse.get(e);
		}
		return null;
	}
	
}
